@@ -1,4 +1,4 @@
-static const char rcsid[] = "$Id: approve.c,v 1.142 2004/04/30 17:00:58 will Exp $";
+static const char rcsid[] = "$Id: approve.c,v 1.146 2004/10/22 00:26:17 will Exp $";
 
 /* The code should compile with either ANSI C or K&R compilers. */
 
@@ -740,6 +740,7 @@ int verbosity;
 {
     char **p, *s;
     int i, j, n;
+    ArgRangePat *arp;
 
     if (verbosity == HELP_BASIC) {
 	if (localinfo.progs.n < 1) {
@@ -903,13 +904,18 @@ int verbosity;
     else
 	(void) printf("max over all args: %ld chars.\n", localinfo.maxlenargs);
 
-    n = StrLastInUse(&localinfo.argpats);
-    if (n >= 0) {
+    arp = (localinfo.argpats.next) ? localinfo.argpats.next :
+						globalinfo.argpats.next;
+    if (arp) {
 	printf("\tUser-entered arguments must match following patterns:\n");
-	for (i = 0; i<=n; i++) {
-	    s = StrEltGetPtr(&localinfo.argpats, i);
-	    if (s)
-		printf("\t\tArg %d must match pattern: %s\n", i, s);
+	for ( ; arp && arp->pat && arp->pat[0]; arp = arp->next) {
+	    if (arp->arg1 == arp->arg2) {
+		fprintf(stderr, "\t\tArg  %d must match pattern: %s\n",
+			    arp->arg1, arp->pat); 
+	    } else {
+		fprintf(stderr, "\t\tArgs %d - %d must match pattern: %s\n",
+			    arp->arg1, arp->arg2, arp->pat);
+	    }
 	}
     }
 
