@@ -1,4 +1,4 @@
-static const char rcsid[] = "$Id: checks.c,v 1.163 2004/04/30 17:00:58 will Exp will $";
+static const char rcsid[] = "$Id: checks.c,v 1.180 2004/10/22 16:00:45 will Exp $";
 /* The code should compile with either ANSI C or K&R compilers. */
 
 /*
@@ -657,10 +657,13 @@ get_encrypted_pw()
 {
     /* Shadow passwords are always used on Sunos 5.x */
     struct spwd *caller_pw;
-    if (!(caller_pw = getspnam(userinfo.caller.pw_name)))
-	return Error(1, 0,
-	    "Failed to obtain shadow password entry for user %s: ",
-	    userinfo.caller.pw_name);
+    if (!(caller_pw = getspnam(userinfo.caller.pw_name))) {
+	authInitErrno = errno;
+	sprintf(authInitMsg1,
+		"Failed to obtain shadow password entry for user %s",
+		userinfo.caller.pw_name);
+	return -1;
+    }
     strcpy(userinfo.encr, caller_pw->sp_pwdp);
     return get_setting();
 }
@@ -676,10 +679,13 @@ get_encrypted_pw()
 
 	struct pr_passwd *caller_pw;
 
-	if (!(caller_pw = getprpwnam(userinfo.caller.pw_name)))
-	    return Error(1, 0,
-		"Failed to obtain Trusted Computing Base entry for user %s: ",
-		userinfo.caller);
+	if (!(caller_pw = getprpwnam(userinfo.caller.pw_name))) {
+	    authInitErrno = errno;
+	    sprintf(authInitMsg1,
+	      "Failed to obtain Trusted Computing Base entry for user %s",
+		userinfo.caller.pw_name);
+	    return -1;
+	}
 	strcpy(userinfo.encr, caller_pw->ufld.fd_encrypt);
 
     } else {
@@ -693,10 +699,13 @@ get_encrypted_pw()
 	if (stat("/.secure/etc/passwd", &st) == 0) {
 	    /* Shadow password file exists; use it */
 	    struct s_passwd *caller_pw;
-	    if (!(caller_pw = getspwnam(userinfo.caller.pw_name)))
-		return Error(1, 0,
-		    "Failed to obtain shadow password entry for user %s: ",
-		    userinfo.caller);
+	    if (!(caller_pw = getspwnam(userinfo.caller.pw_name))) {
+		authInitErrno = errno;
+		sprintf(authInitMsg1,
+		    "Failed to obtain shadow password entry for user %s",
+		    userinfo.caller.pw_name);
+		return -1;
+	    }
 	    strcpy(userinfo.encr, caller_pw->pw_passwd);
 	} else {
 
@@ -717,19 +726,24 @@ get_encrypted_pw()
     struct passwd *caller_pw;
     struct spwd *caller_ps;
 
-    if (!(caller_pw = getpwnam(userinfo.caller.pw_name)))
-	return Error(0, 0, "No password entry for user %s.\n",
+    if (!(caller_pw = getpwnam(userinfo.caller.pw_name))) {
+	sprintf(authInitMsg1, "No password entry for user %s.\n",
 		userinfo.caller.pw_name);
+	return -1;
+    }
 
     /* SCO 3.2v4 has "x" in password field to indicate shadow password
      * file has to be consulted.
      */
     if (strcmp(caller_pw->pw_passwd, "x") == 0) {
 	/* Shadow password in use... */
-	if (!(caller_ps = getspnam(userinfo.caller.pw_name)))
-	    return Error(1, 0,
-	    "Failed to obtain shadow password entry for user %s: ",
-	    userinfo.caller.pw_name);
+	if (!(caller_ps = getspnam(userinfo.caller.pw_name))) {
+	    authInitErrno = errno;
+	    sprintf(authInitMsg1,
+		"Failed to obtain shadow password entry for user %s",
+		userinfo.caller.pw_name);
+	    return -1;
+	}
 	strcpy(userinfo.encr, caller_ps->sp_pwdp);
 
     } else {
@@ -749,19 +763,24 @@ get_encrypted_pw()
     struct passwd *caller_pw;
     struct pr_passwd *caller_prpass;
 
-    if (!(caller_pw = getpwnam(userinfo.caller.pw_name)))
-	return Error(0, 0, "No password entry for user %s.\n",
+    if (!(caller_pw = getpwnam(userinfo.caller.pw_name))) {
+	sprintf(authInitMsg1, "No password entry for user %s.\n",
 		userinfo.caller.pw_name);
+	return -1;
+    }
 
     /* Digital Unix has "x" in password field to indicate shadow password
      * file has to be consulted.
      */
     if (strcmp(caller_pw->pw_passwd, "x") == 0) {
 	/* Protected (shadow) password in use... */
-	if (!(caller_prpass = getprpwnam(userinfo.caller.pw_name)))
-	    return Error(1, 0,
-	    "Failed to obtain shadow password entry for user %s: ",
+	if (!(caller_prpass = getprpwnam(userinfo.caller.pw_name))) {
+	    authInitErrno = errno;
+	    sprintf(authInitMsg1,
+		"Failed to obtain shadow password entry for user %s",
 	    userinfo.caller.pw_name);
+	    return -1;
+	}
 	strcpy(userinfo.encr, caller_prpass->ufld.fd_encrypt);
 
     } else {
@@ -790,9 +809,11 @@ get_encrypted_pw()
 	    if (errno == ENOENT || errno == 0) {
 		strcpy(userinfo.encr, userinfo.caller.pw_passwd);
 	    } else {
-		return Error(1, 0,
-		    "Failed to obtain shadow password entry for user %s: ",
+		authInitErrno = errno;
+		sprintf(authInitMsg1,
+		    "Failed to obtain shadow password entry for user %s",
 		    userinfo.caller.pw_name);
+		return -1;
 	    }
         } else {
 	    /* got shadow entry */
@@ -819,8 +840,9 @@ get_encrypted_pw()
      */
     strcpy(userinfo.encr, userinfo.caller.pw_passwd);
     if (strcmp(userinfo.encr, "*") == 0) {
-	return Error(0, 0, "Can't read encrypted password for user %s",
-			userinfo.caller.pw_name);
+	sprintf(authInitMsg1, "Can't get encrypted password, \
+or there is no password, for user %s", userinfo.caller.pw_name);
+	return -1;
     }
 
     return get_setting();
@@ -839,8 +861,9 @@ get_encrypted_pw()
      */
     strcpy(userinfo.encr, userinfo.caller.pw_passwd);
     if (strcmp(userinfo.encr, "*") == 0) {
-	return Error(0, 0, "Can't read encrypted password for user %s",
-			userinfo.caller.pw_name);
+	sprintf(authInitMsg1, "Can't get encrypted password, \
+or there is no password, for user %s", userinfo.caller.pw_name);
+	return -1;
     }
 
     return get_setting();
@@ -948,15 +971,35 @@ char *cmd;
 {
     char file[MAXPATHLEN];
     struct stat st;
-    int l, istat, file_exists, got_auth;
+    int l, istat, file_exists, got_auth, err, timestamp_creatok;
     int timed_out = 0;
     int status;
     pid_t pid, child;
+    char mkdirMsg[1000];
     char *authuser;
 
     if (!localinfo.authinfo.required)
 	return 0;			/* don't need authentication */
 
+    if (*authInitMsg1 || *authInitMsg2) {
+	/* A message was generated during authentication initialization,
+	 * way back before the super.tab file was read.  It was only
+	 * relevant if the command required authorization... well, this
+	 * command requires user authentication, so print the message now.
+	 */
+	if (*authInitMsg1) {
+	    if (authInitErrno) {
+		errno = authInitErrno;
+		Error(1, 0, "%s: ", authInitMsg1);
+	    } else {
+		Error(0, 0, "%s", authInitMsg1);
+	    }
+	}
+	if (*authInitMsg2) {
+	    Error(0, 0, "%s", authInitMsg2);
+	}
+    }
+
     /* Create or update the timestamp file even if the lifetime is 0
      * (always ask for password).  We do this because the user may
      * execute _another_ command which has a password expiry > 0,
@@ -965,7 +1008,8 @@ char *cmd;
      */
     child = fork();
     if (child == -1) {
-	return Error(1, 0, "Failed to create child for timestamp processing: ");
+	Error(1, 0, "Failed to create child for timestamp processing: ");
+	return -1;
 
     } else if (child > 0) {
 	/* In parent -- wait to see if the child succeeded */
@@ -976,24 +1020,35 @@ char *cmd;
 		globalinfo.log.pid = -1;
 	    } else {
 		Error(0, 0,
-		    "wait() surprised! check_pass() received pid %d;\n\t\
+		    "Wait() surprised! check_pass() received pid %d;\n\t\
 expected child pid = %d; waiting for correct pid...\n", pid, child);
 	    }
 	}
-	if (pid == -1)    
-	    return Error(1, 0, "waiting for timestamp creation process: ");
-	else if (status == 0)
+	if (pid == -1) {
+	    /* wait() failed */
+	    Error(1, 0, "Waiting for timestamp creation process: ");
+	    return -1;
+	} else if (status == 0) {
+	    /* child succeeded */
 	    return 0;
-	else
-	    return Error(0, 0, "Authentication or timestamp creation failed\n");
+	} else if (status != 0) {
+	    /* child failed to authenticate user */
+	    Error(0, 0, "Authentication failed\n");
+	    return -1;
+	}
 	/* NOTREACHED */
     }
 	
     /*
      * If here, must be in child.
+     * Unless this command specifies a timeout > 0, don't generate
+     * error messages if we fail to store timestamp info.
+     * We'll use exit code 1 for failure to authenticate;
+     * otherwise we'll exit 0 (success).
      */
-    if (child != 0) Error(0, 1,
-    			"Internal error: child=%d; should be 0!\n", child);
+    if (child != 0) {
+	Error(0, 1, "Internal error: child=%d; should be 0!\n", child);
+    }
 
     /*
      * setuid, then make and/or test the directory
@@ -1004,39 +1059,87 @@ expected child pid = %d; waiting for correct pid...\n", pid, child);
 	*localinfo.group = '\0';
 	*localinfo.u_g = '\0';
 	if (set_u_g() == -1) {
-	    return Error(1, 0,
-	    "failed to setuid %s before setting timestamp file: ",
-	    localinfo.user);
+	    if (localinfo.authinfo.timeout > 0) {
+		Error(1, 0,
+		"failed to setuid to user=%s before setting timestamp file: ",
+		    localinfo.user);
+	    }
+	    exit(2);
 	}
     }
     /* Make the timestamp directory name */
+    timestamp_creatok = 1;
+    mkdirMsg[0] = '\0';
     if (!makedirname(TIMESTAMP_DIR,
 		    globalinfo.authinfo.perhost ? userinfo.hostname : "",
-		    file))
-	return -1;
+		    file, &err, mkdirMsg)) {
+	timestamp_creatok = 0;
+	if (localinfo.authinfo.timeout > 0 && mkdirMsg[0]) {
+	    if (err) {
+		errno = err;
+		Error(1, 0, "Warning: can't record timestamp: %s: ",
+			mkdirMsg);
+	    } else {
+		Error(0, 0, "Warning: can't record timestamp: %s",
+			mkdirMsg);
+	    }
+	}
+    }
 
     /* Make the timestamp directory */
-    if (makedir(file) == -1)
-	return -1;
+    if (timestamp_creatok) {
+	mkdirMsg[0] = '\0';
+	if (makedir(file, &err, mkdirMsg) == -1) {
+	    timestamp_creatok = 0;
+	    if (localinfo.authinfo.timeout > 0 && mkdirMsg[0]) {
+		if (err) {
+		    errno = err;
+		    Error(1, 0,
+			"Warning: can't record timestamp: %s: ",
+			mkdirMsg);
+		} else {
+		    Error(0, 0,
+			"Warning: can't record timestamp: %s",
+			mkdirMsg);
+		}
+	    }
+	}
+    }
 
     /* Make the file in the timestamp directory */
-    l = strlen(file) + 1 + strlen(userinfo.caller.pw_name);
-    if (l >= MAXPATHLEN)
-	return Error(1, 0,
-	"Can't create timestamp file: it would exceed MAXPATHLEN = %d\n",
-	MAXPATHLEN);
-    strcat(file, "/");
-    strcat(file, userinfo.caller.pw_name);
+    if (timestamp_creatok) {
+	l = strlen(file) + 1 + strlen(userinfo.caller.pw_name);
+	if (l >= MAXPATHLEN) {
+	    if (localinfo.authinfo.timeout > 0) {
+		Error(1, 0,
+		"Can't create timestamp file: would exceed MAXPATHLEN = %d\n",
+		MAXPATHLEN);
+	    }
+	    timestamp_creatok = 0;
+	}
+    }
 
-    istat = stat(file, &st);
-    if (istat != 0 && errno != ENOENT)
-	return Error(1, 0, "Failed to stat timestamp file `%s': ", file);
+    if (timestamp_creatok) {
+	strcat(file, "/");
+	strcat(file, userinfo.caller.pw_name);
 
-    file_exists = (istat == 0);
-    if (file_exists)
-	timed_out = (localinfo.authinfo.timeout < 1) ||
-		    ((time(NULL)-st.st_mtime) >
-		    localinfo.authinfo.timeout*60);
+	istat = stat(file, &st);
+	if (istat != 0 && errno != ENOENT) {
+	    if (localinfo.authinfo.timeout > 0) {
+		Error(1, 0, "Failed to stat timestamp file `%s': ", file);
+	    }
+	    timestamp_creatok = 0;
+	}
+    }
+
+    if (timestamp_creatok) {
+	file_exists = (istat == 0);
+	if (file_exists) {
+	    timed_out = (localinfo.authinfo.timeout < 1) ||
+			((time(NULL)-st.st_mtime) >
+			localinfo.authinfo.timeout*60);
+	}
+    }
 
     got_auth=0;
     if (localinfo.authinfo.user[0]) {
@@ -1046,14 +1149,19 @@ expected child pid = %d; waiting for correct pid...\n", pid, child);
     } else {
 	authuser = userinfo.caller.pw_name;	/* default: caller's pw */
     }
-    if (!file_exists || timed_out) {
+    if (!timestamp_creatok || !file_exists || timed_out) {
 	switch (localinfo.authinfo.method) {
 	case SUPER_AUTH_PASSWORD:
 	    got_auth = (get_password(cmd, userinfo.caller.pw_name, authuser,
 				    userinfo.salt, userinfo.encr) == 1);
 	    break;
 	case SUPER_AUTH_PAM:
+#if WITH_PAM
 	    got_auth = (get_pam(cmd, userinfo.caller.pw_name, authuser) == 1);
+#else
+	    Error(0, 1, "Auth method is PAM, but this copy of super \
+was compiled without PAM support!");
+#endif
 	    break;
 	default:
 	    Error(0, 1, "Internal error in check_auth: \
@@ -1074,7 +1182,12 @@ unknown auth method %d!\n", localinfo.authinfo.method);
 
      * Otherwise create the file (unlink it first if it exists).
      */
-    if (file_exists && !(got_auth || localinfo.authinfo.renewtime)) {
+    if (!timestamp_creatok) {
+	/* do nothing: we failed in setting up the timestamp file,
+	 * so don't try to update it.
+	 */
+
+    } else if (file_exists && !(got_auth || localinfo.authinfo.renewtime)) {
 	if (st.st_uid != geteuid())
 	    return Error(0, 0,
 	"Timestamp file `%s' is owned by uid=%d, but expected owner=%d.\n\
