@@ -1,4 +1,4 @@
-static char rcsid[] = "$Id: error.c,v 1.67 2000/08/20 05:36:55 will Exp $";
+static const char rcsid[] = "$Id: error.c,v 1.78 2004/05/18 02:48:47 will Exp $";
 
 /*
  *	Copyright (c) 1993 by California Institute of Technology.
@@ -83,13 +83,10 @@ static char rcsid[] = "$Id: error.c,v 1.67 2000/08/20 05:36:55 will Exp $";
 
  * If HAVE_SYSLOG_H is defined, and the caller sets error_syslog != 0,
  *	then the message is passed to [r]syslog(), at priority error_priority,
- *	using facility error_facility.  Note that depending on the
- *	"#if USE_RSYSLOG" setting, we compile with either rsyslog (the default)
- *	or syslog.
- *	If rsyslog is used, we send to the syslog facility of error_rlog_host
- *	(default ourselves), to provide networked syslog messages.
- *	Otherwise we compile with standard syslog routines.
- *	In this case, the fmt string and the printf output must be less
+ *	using facility error_facility.  We compile with rsyslog, but if the
+ *	remote host argument is NULL at the time we open the log, we use the
+ *	syslog routines instead.
+ *	The fmt string and the printf output must be less
  *	than MAXPRINT characters each. This is because syslog() accepts a
  *	printf-style variadic argument list, but it doesn't have a va_list
  *	version.  Therefore we print into a string and pass that onto syslog().
@@ -163,24 +160,42 @@ int error_fflush = 1;
 #endif
 #endif
 
-#if USE_RSYSLOG
 void ropenlog P__(( char *ident, int logopt, int facility, char *host ));
 void rsyslog P__(( unsigned int level, char *fmt, ... ));
 
-#define OpenLog(prog, opt, fac) ropenlog((prog), (opt), (fac), error_rlog_host)
-#define SysLog(pri, buf) rsyslog((pri), (buf))
-
-#else
-
-#define OpenLog(prog, opt, fac) openlog((prog), (opt), (fac))
-#define SysLog(pri, buf) syslog((pri), (buf))
-
-#endif
-
 int error_priority = SYSLOG_PRIORITY;
 int error_facility = SYSLOG_FACILITY;
 int openlog_done = 0;
 
+static int using_rsyslog = 0;
+
+void
+OpenLog(prog, opt, fac)
+char *prog;
+int opt;
+int fac;
+{
+    if (error_rlog_host != NULL) {
+	ropenlog(prog, opt, fac, error_rlog_host);
+	using_rsyslog = 1;
+    } else {
+	openlog(prog, opt, fac);
+	using_rsyslog = 0;
+    }
+}
+
+void
+SysLog(pri, buf)
+int pri;
+char *buf;
+{
+    if (using_rsyslog) {
+	rsyslog(pri, "%s", buf);
+    } else {
+	syslog(pri, "%s", buf);
+    }
+}
+
 #endif
 
 static int uid = -1;
@@ -485,8 +500,8 @@ va_dcl
 	show_perror = va_arg(ap, int);
 	die = va_arg(ap, int);
 	orig_fmt = va_arg(ap, char *);
-	if (!openlog_done && error_prog) {
-	    OpenLog(error_prog, 0, error_facility);
+	if (!openlog_done) {
+	    OpenLog(error_prog ? error_prog : "", 0, error_facility);
 	    openlog_done = 1;
 	}
 	sprintf(newfmt, "(%s) ", error_user ? error_user : user);
