@@ -1,4 +1,4 @@
-static const char rcsid[] = "$Id: super.c,v 1.578 2004/06/18 16:18:20 will Exp $";
+static const char rcsid[] = "$Id: super.c,v 1.592 2004/10/21 21:35:25 will Exp $";
 /* The code should compile with either ANSI C or K&R compilers. */
 
 /*
@@ -94,6 +94,8 @@ GlobalInfo globalinfo = {
 	0,		/* mask (umask) */
 	MAXLEN1ARG,	/* maximum length of a single argument */
 	MAXLENARGS,	/* maximum length of all arguments, combined */
+	{-1,-1},	/* min, max number of args (-1 = no limit) */
+	{NULL,0,0,NULL},		/* argpats */
 
 	 {		/* Global password requirements */
 		0,	/* bool: is authentication required? */
@@ -145,6 +147,12 @@ GlobalInfo globalinfo = {
 /* The list of currently open files */
 FileList *currfile = NULL;
 
+char authInitMsg1[1024] = "";	/* message from auth init, if any */
+char authInitMsg2[1024] = "";	/* suppl message from auth init, if any */
+int authInitErrno = 0;		/* set to !0, if there's an errno to go
+				 * with the authInitMsg1.
+				 */
+
 /* The struct of what things we've matched */
 Conditions matches;
 
@@ -236,6 +244,7 @@ char **argv;
 	return 1;
     init_globalinfo();		/* Initialize globalinfo struct. */
     init_localinfo();		/* Initialize localinfo struct. */
+    add_variable("PATTERNS", "regex");	/* default pattern lookup */
 
     add_builtin_variables();	/* Define variables that describe this system */
 
@@ -245,11 +254,12 @@ char **argv;
 	*userinfo.encr = '\0';
 	*userinfo.salt = '\0';
 	if (geteuid() != 0) {
-	    Error(0, 0, "\tWon't be able to do password-checking.  \
-Note: this copy of super is not \
-running setuid-root, so it can't check passwords on modern Unix systems.\n\n");
+	    sprintf(authInitMsg2, "\tWon't be able to do password-checking.  \
+Note: this copy of super is not running setuid-root, \
+so it can't check passwords on modern Unix systems.\n\n");
 	} else {
-	    Error(0, 0, "\tWon't be able to do authentication (password-checking, or whatnot).\n\n");
+	    sprintf(authInitMsg2, "\tWon't be able to do authentication \
+(password-checking, or whatnot).\n\n");
 	}
     }
 
@@ -626,7 +636,7 @@ char **o_file;
 		    return Error(0, 0, "No machine specified after option -M\n");
 		}
 		if (strlen(mhost) > sizeof(userinfo.hostname)-1)
-		    Error(1, 1, "Machine name too long -- max allowed is %d\n",
+		    Error(0, 1, "Machine name too long -- max allowed is %d\n",
 				sizeof(userinfo.hostname)-1);
 		strcpy(userinfo.hostname, mhost);
 		if (canonicalize_hostname(userinfo.hostname,
@@ -829,6 +839,7 @@ int n_builtin;
     char *s, *t, **p, **sp;
     char *cdpath;
     int isglobal, j, iarg;
+    ArgRangePat  *argpats, *arp;
 
     fprintf(stderr,
 	  "==============================================================\n");
@@ -868,9 +879,12 @@ int n_builtin;
 	iarg = sp - arglist;
 	(void) fprintf(stderr, "\tArgv[%d]:  <%s>\n", iarg, *sp);
 	if (iarg > n_builtin) {
-	    s = StrEltGetPtr(&localinfo.argpats, iarg-n_builtin);
-	    if (s)
-		fprintf(stderr, "\t\tMust match pattern: %s\n", s);
+	    argpats = (localinfo.argpats.next) ? localinfo.argpats.next :
+						    globalinfo.argpats.next;
+	    for (arp=ARnext(argpats, iarg-n_builtin); arp;
+				    arp = ARnext(arp->next, iarg-n_builtin)) {
+		fprintf(stderr, "\t\tMust match pattern: %s\n", arp->pat);
+	    }
 	}
     }
 
@@ -1048,16 +1062,16 @@ Command <%s:%-.500s>; uid=%d, euid=%d: ",
 	    user, pass->pw_gid, user, cmd, getuid(), geteuid());
 
     if ((i=setgid(pass->pw_gid)) == -1) {
-	Error(1, 1, "Can't set gid to %d\n", pass->pw_gid);
+	Error(1, 1, "Can't set gid to %d: ", pass->pw_gid);
     } else if ((j=getgid()) != pass->pw_gid) {
 	Error(1, 1, "setgid(gid=%d) returned %d, but getgid() returned %d!",
 	    pass->pw_gid, i, j);
     }
 
     if ((i=setuid(pass->pw_uid)) == -1) {
-	Error(1, 1, "Can't set uid to %d\n", pass->pw_uid);
+	Error(1, 1, "Can't set uid to %d: ", pass->pw_uid);
     } else if ((j=getuid()) != pass->pw_uid) {
-	Error(1, 1, "setuid(uid=%d) returned %d, but getuid() returned %d!",
+	Error(0, 1, "setuid(uid=%d) returned %d, but getuid() returned %d!",
 	    pass->pw_uid, i, j);
     }
 }
@@ -1160,6 +1174,7 @@ int *n_builtin;		/* returned w/ number of args in path_plus */
     int nuser, nalloc, iarg, nargs, nxtra;
     char **arglist, **ap;
     char *s;
+    ArgRangePat  *argpats, *arp;
 
     /* Count user-entered args. */
     for (ap = argv; *ap; )
@@ -1188,11 +1203,14 @@ int *n_builtin;		/* returned w/ number of args in path_plus */
 
     /* Check that each user-entered argument matches its pattern, if given */
     for (iarg = 1; iarg <= nargs; iarg++) {
-	s = StrEltGetPtr(&localinfo.argpats, iarg);
-	if (s && match_pattern(0, 1, argv[iarg], s) != 1)
-	    (void) Error(0, 2,
-		"Your argument #%d <%-.500s> must match pattern <%s>\n",
-		iarg, argv[iarg], s);
+	argpats = (localinfo.argpats.next) ? localinfo.argpats.next :
+						    globalinfo.argpats.next;
+	for (arp=ARnext(argpats, iarg); arp; arp = ARnext(arp->next, iarg)) {
+	    if (arp->pat && match_pattern(0, 1, argv[iarg], arp->pat) != 1)
+		(void) Error(0, 2,
+		    "Your argument #%d <%-.500s> must match pattern <%s>\n",
+		    iarg, argv[iarg], arp->pat);
+	}
     }
 
     /* Start off with space for user-entered args + 100 args in super.tab.
@@ -1203,7 +1221,7 @@ int *n_builtin;		/* returned w/ number of args in path_plus */
     arglist = (char **) malloc(sizeof(char *) * nalloc);
     if (!arglist)
 	(void) Error(1, 2, 
-	    "failed to malloc space for %d ptrs\n", nalloc);
+	    "failed to malloc space for %d ptrs: ", nalloc);
 
     /* Leave room for two extra args at front, in case we are handling
      * the "#! interpreter [opt]" file for OS's that don't support it.
@@ -1238,7 +1256,7 @@ int *n_builtin;		/* returned w/ number of args in path_plus */
 	    newarglist = (char **) realloc((void *) arglist, nalloc);
 	    if (!newarglist)
 		(void) Error(1, 2, 
-		    "failed to realloc space for %d ptrs\n", nalloc);
+		    "failed to realloc space for %d ptrs: ", nalloc);
 	    ap = newarglist + (ap - arglist);
 	    arglist = newarglist;
 	}
@@ -1564,7 +1582,10 @@ init_localinfo()
      */
     localinfo.ngroups = GROUPS_NOTSET;
     localinfo.groups_added = 0;
-    StrInit(&localinfo.argpats);
+    localinfo.argpats.pat = NULL;
+    localinfo.argpats.arg1 = -1;
+    localinfo.argpats.arg2 = -1;
+    localinfo.argpats.next = NULL;
 }
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
