@@ -16,7 +16,7 @@
 /* ==================================================================== */
 
 #include "localsys.h"
-#include "hsearch.h"
+#include "s_hsearch.h"
 #include "version.h"
 
 /* ==================================================================== */
